package ch.fritteli.gombaila.domain.printer;

import io.vavr.collection.Seq;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

record FormattedLine(@NotNull Seq<String> tokens, @Nullable String comment, int indentationLevel,
                     @NotNull WidthMap widthMap) implements Line {
    @Override
    public void updateWithMap() {
        this.tokens
                .forEachWithIndex((token, index) -> {
                    this.widthMap.updateWidthForTokenIndex(index, token.length());
                });
    }

    @NotNull
    @Override
    public String toString() {
        final Seq<String> lineElements;
        if (this.comment == null) {
            lineElements = this.tokens.dropRight(1)
                    .zipWithIndex()
                    .map(tuple -> tuple.map2(this.widthMap::getTokenWidthByIndex))
                    .map(tuple -> {
                        final String token = tuple._1();
                        final int width = tuple._2();
                        return String.format("%-" + width + "s", token);
                    })
                    .appendAll(this.tokens.lastOption());
        } else {
            lineElements = this.tokens.zipWithIndex()
                    .map(tuple -> tuple.map2(this.widthMap::getTokenWidthByIndex))
                    .map(tuple -> {
                        final String token = tuple._1();
                        final int width = tuple._2();
                        return String.format("%-" + width + "s", token);
                    })
                    .append(";")
                    .append(this.comment);
        }
        final String line = lineElements.mkString(" ");
        final int indents = this.widthMap.getIndentationSize() * this.indentationLevel;
        if (indents == 0) {
            return line;
        }
        return ("%-" + indents + "s").formatted("") + line;
    }
}
