package ch.fritteli.gombaila.domain.printer;

import io.vavr.collection.List;
import lombok.NonNull;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;

public class Printer {
    private static final int INDENTATION_SIZE = 4;
    private final java.util.List<Printable> printables = new ArrayList<>();
    private WidthMap widthMap = new WidthMap(INDENTATION_SIZE);
    private int indentationLevel = 0;

    public void increaseIndent() {
        this.indentationLevel++;
    }

    public void decreaseIndent() {
        this.indentationLevel--;
    }

    public void resetIndent() {
        this.indentationLevel = 0;
    }

    public void comment(@NotNull final String line) {
        this.printables.add(new RawLine("; %s".formatted(line), this.indentationLevel, this.widthMap));
    }

    public void section(@NotNull final String sectionName) {
        this.resetIndent();
        this.widthMap = new WidthMap(INDENTATION_SIZE);
        this.printables.add(new RawLine("section %s".formatted(sectionName), this.indentationLevel, this.widthMap));
    }

    public void label(@NotNull final String label) {
        this.resetIndent();
        this.widthMap = new WidthMap(INDENTATION_SIZE);
        this.printables.add(new RawLine("%s:".formatted(label), this.indentationLevel, this.widthMap));
        this.increaseIndent();
    }

    public void blank() {
        this.printables.add(new RawLine("", 0, this.widthMap));
    }

    public void line(@NonNull final Object... instructionTokens) {
        final List<String> tokens = List.of(instructionTokens)
                .map(String::valueOf);
        this.printables.add(new FormattedLine(tokens, null, this.indentationLevel, this.widthMap));
    }

    public void commentedLine(@NotNull final String comment, @NotNull final Object... instructionTokens) {
        final List<String> tokens = List.of(instructionTokens)
                .map(String::valueOf);
        this.printables.add(new FormattedLine(tokens, comment, this.indentationLevel, this.widthMap));
    }

    public void rawLine(@NotNull final String line) {
        this.printables.add(new RawLine(line, this.indentationLevel, this.widthMap));
    }

    @Override
    public String toString() {
        this.printables.forEach(Printable::updateWithMap);
        return List.ofAll(this.printables).mkString("\n");
    }
}
