package ch.fritteli.gombaila.domain.printer;

import io.vavr.collection.HashMap;
import io.vavr.collection.Map;
import lombok.Getter;
import org.jetbrains.annotations.NotNull;

public class WidthMap {
    @Getter
    private final int indentationSize;
    @NotNull
    private Map<Integer, Integer> widthByTokenIndex = HashMap.empty();

    public WidthMap(final int indentationSize) {
        this.indentationSize = indentationSize;
    }

    int getMaxLineLengthWithoutComment() {
        return this.widthByTokenIndex.values()
                .sum()
                .intValue();
    }

    int getTokenWidthByIndex(final int tokenIndex) {
        return this.widthByTokenIndex.get(tokenIndex).getOrElse(0);
    }

    void updateWidthForTokenIndex(final int tokenIndex, final int width) {
        if (!this.hasBiggerWidthAt(tokenIndex, width)) {
            this.widthByTokenIndex = this.widthByTokenIndex.put(tokenIndex, width);
        }
    }

    private boolean hasBiggerWidthAt(final int tokenIndex, final int width) {
        return this.widthByTokenIndex.get(tokenIndex)
                .exists(existingWidth -> existingWidth > width);
    }
}
